@@ -15,15 +15,20 @@ class MPCache(object):
     path that much be global to all process using this cache. Cache content is
     structured as a dictionary and content is pickled before saving to disk.
     """
-    def __init__(self, cache_id, cache_type, global_path):
+    def __init__(self, cache_id, cache_type, global_path,
+                 use_single_file=False):
         """
-        @param cache_id: a unique name for this cache.
-        @param cache_type: a name given to this type of cache.
-        @param global_path: path shared across all processes using this cache.
+        @param cache_id: A unique name for this cache.
+        @param cache_type: A name given to this type of cache.
+        @param global_path: Path shared across all processes using this cache.
+        @param use_single_file: By default each key is stored as its own file
+                                but if this is set to True a single file is
+                                used to store all keys.
         """
         self.cache_id = cache_id
         self.cache_type = cache_type
         self.global_path = global_path
+        self.file_per_key = not use_single_file
 
     @cached_property
     def _global_cache_lock(self):
@@ -51,8 +56,13 @@ class MPCache(object):
         path = os.path.join(self.global_path, 'caches', self.cache_type,
                             self.cache_id)
         with self._global_cache_lock:
-            if not os.path.isdir(os.path.dirname(path)):
-                os.makedirs(os.path.dirname(path))
+            if self.file_per_key:
+                _dir = path
+            else:
+                _dir = os.path.dirname(path)
+
+            if not os.path.isdir(_dir):
+                os.makedirs(_dir)
 
         return path
 
@@ -83,6 +93,9 @@ class MPCache(object):
         path = self._cache_path
         with self._cache_lock:
             log.debug("load from cache '%s' (key='%s')", path, key)
+            if self.file_per_key:
+                return self._get_unsafe(os.path.join(path, key))
+
             contents = self._get_unsafe(path)
             if contents:
                 return contents.get(key)
@@ -102,14 +115,20 @@ class MPCache(object):
             return
 
         with self._cache_lock:
-            contents = self._get_unsafe(path)
-            if contents:
-                contents[key] = value
+            if self.file_per_key:
+                path = os.path.join(path, key)
+                log.debug("saving to cache '%s' (key=%s)", path, key)
+                contents = value
             else:
-                contents = {key: value}
+                contents = self._get_unsafe(path)
+                if contents:
+                    contents[key] = value
+                else:
+                    contents = {key: value}
+
+                log.debug("saving to cache '%s' (key=%s, items=%s)", path, key,
+                          len(contents))
 
-            log.debug("saving to cache '%s' (key=%s, items=%s)", path, key,
-                      len(contents))
             with open(path, 'wb') as fd:
                 pickle.dump(contents, fd)
 
